@@ -1154,6 +1154,14 @@ void EnzymeCopyMetadata(LLVMValueRef inst1, LLVMValueRef inst2) {
   cast<Instruction>(unwrap(inst1))
       ->copyMetadata(*cast<Instruction>(unwrap(inst2)));
 }
+void EnzymeCopyAlignment(LLVMValueRef inst1, LLVMValueRef inst2) {
+  cast<AllocaInst>(unwrap(inst1))
+      ->setAlignment(cast<AllocaInst>(unwrap(inst2))->getAlign());
+}
+void EnzymeTakeName(LLVMValueRef inst1, LLVMValueRef inst2) {
+  unwrap(inst1)->takeName(unwrap(inst2));
+}
+
 LLVMMetadataRef EnzymeAnonymousAliasScopeDomain(const char *str,
                                                 LLVMContextRef ctx) {
   MDBuilder MDB(*unwrap(ctx));
@@ -1433,7 +1441,7 @@ void EnzymeFixupBatchedJuliaCallingConvention(LLVMValueRef F_C) {
       } else if (attr.isStringAttribute() &&
                  attr.getKindAsString() == "enzymejl_returnRoots_v") {
         sretv = true;
-        kind = "enzymejl_returnRoots_v";
+        kind = "enzymejl_returnRoots";
         value = attr.getValueAsString();
       } else {
         NewAttrs = NewAttrs.addAttribute(
@@ -1457,6 +1465,7 @@ void EnzymeFixupBatchedJuliaCallingConvention(LLVMValueRef F_C) {
         }
       }
     }
+    assert(!sretv);
     types.push_back(T);
   }
   if (changed.size() == 0)
@@ -1552,17 +1561,17 @@ void EnzymeFixupBatchedJuliaCallingConvention(LLVMValueRef F_C) {
                   attr.getKindAsString() == "enzyme_sret_v") {
                 sretv = true;
                 kind = "enzyme_sret";
-                attr.getValueAsString();
+                value = attr.getValueAsString();
               } else if (attr.isStringAttribute() &&
                          attr.getKindAsString() == "enzymejl_returnRoots_v") {
                 sretv = true;
                 kind = "enzymejl_returnRoots";
-                attr.getValueAsString();
+                value = attr.getValueAsString();
               } else if (attr.isStringAttribute() &&
                          attr.getKindAsString() == "enzymejl_rooted_typ_v") {
                 sretv = true;
                 kind = "enzymejl_rooted_typ_v";
-                attr.getValueAsString();
+                value = attr.getValueAsString();
               }
             }
             for (unsigned i = 0; i < AT->getNumElements(); i++) {
@@ -1583,17 +1592,20 @@ void EnzymeFixupBatchedJuliaCallingConvention(LLVMValueRef F_C) {
             attr.getKindAsString() == "enzyme_sret_v") {
           NewAttrs = NewAttrs.addAttribute(
               F->getContext(), AttributeList::FirstArgIndex + vals.size(),
-              Attribute::get(F->getContext(), "enzyme_sret"));
+              Attribute::get(F->getContext(), "enzyme_sret",
+                             attr.getValueAsString()));
         } else if (attr.isStringAttribute() &&
                    attr.getKindAsString() == "enzymejl_returnRoots_v") {
           NewAttrs = NewAttrs.addAttribute(
               F->getContext(), AttributeList::FirstArgIndex + vals.size(),
-              Attribute::get(F->getContext(), "enzymejl_returnRoots"));
+              Attribute::get(F->getContext(), "enzymejl_returnRoots",
+                             attr.getValueAsString()));
         } else if (attr.isStringAttribute() &&
                    attr.getKindAsString() == "enzymejl_rooted_typ_v") {
           NewAttrs = NewAttrs.addAttribute(
               F->getContext(), AttributeList::FirstArgIndex + vals.size(),
-              Attribute::get(F->getContext(), "enzymejl_rooted_typ"));
+              Attribute::get(F->getContext(), "enzymejl_rooted_typ",
+                             attr.getValueAsString()));
         } else {
           NewAttrs = NewAttrs.addAttribute(
               F->getContext(), AttributeList::FirstArgIndex + vals.size(),
@@ -1754,6 +1766,9 @@ bool needsReRooting(llvm::Argument *arg, bool &anyJLStore,
   for (auto &&[I, cur, byteOffset] : findAllUsersOf(arg)) {
     assert(I->getParent()->getParent() == arg->getParent());
 
+    if (isa<ICmpInst>(I)) {
+      continue;
+    }
     if (isa<LoadInst>(I)) {
       continue;
     }
@@ -2666,7 +2681,8 @@ void EnzymeFixupJuliaCallingConvention(LLVMValueRef F_C,
       if (jlvalue) {
         auto ld = B.CreateLoad(ty, gep);
         auto SI = B.CreateStore(ld, val);
-        PostCacheStore(SI, B);
+        if (val->getType()->getPointerAddressSpace() == 10)
+          PostCacheStore(SI, B);
       } else {
         copyNonJLValueInto(B, ty, ty, val, {}, ty, gep, {},
                            /*shouldZero*/ false);
